//#include "metalShaderTypes.h"

typedef struct
{
	float blendValue;
	float speed;
} fxVars;


float makePoint(float x,float y,float fx,float fy,float sx,float sy,float t)
{
    float xx=x+sin(t*fx)*sx;
    float yy=y+cos(t*fy)*sy;
    return 1.0/sqrt(xx*xx+yy*yy);
}

vertex fxVertexOut vertexFunc(uint vertexID [[ vertex_id ]]
							  ,const device fxShaderVerts* in[[ buffer(0) ]]
							  ,texture2d<half> inputTex0 [[ texture(0) ]]
							  ,texture2d<half> inputTexMask [[ texture(1) ]]
							  ,constant fxGeneralUniforms& u[[ buffer(1) ]]
							  ,constant fxVars& vars[[ buffer(2)]])
{
    fxVertexOut out;

	out.normPos=in[vertexID].pos.xy*vec2(.5,.5f)+vec2(.5,.5);
	out.position=in[vertexID].pos;

    out.tex1=in[vertexID].tex1;
	out.texMask=in[vertexID].texMask;
	out.texSize=float2(inputTex0.get_width(),inputTex0.get_height());
    return out;
}

fragment float4 fragmentFunc(fxVertexOut in [[stage_in]]
							 ,texture2d<half> inputTex [[ texture(0) ]]
							 ,texture2d<half> inputTexMask [[ texture(1) ]]
							 ,constant fxGeneralUniforms& u[[ buffer(0) ]]
							 ,constant fxVars& vars[[ buffer(1)]])
{
	
    vec2 p=in.tex1*2.0-vec2(1.,1.);
	  
	float time=u.localTime*vars.speed;

	p=p*.70;

	float x=p.x;
	float y=p.y;

	float a=
	makePoint(x,y,3.3,2.9,0.3,0.3,time);
	a=a+makePoint(x,y,1.9,2.0,0.4,0.4,time);
	a=a+makePoint(x,y,0.8,0.7,0.4,0.5,time);
	a=a+makePoint(x,y,2.3,0.1,0.6,0.3,time);
	a=a+makePoint(x,y,0.8,1.7,0.5,0.4,time);
	a=a+makePoint(x,y,0.3,1.0,0.4,0.4,time);
	a=a+makePoint(x,y,1.4,1.7,0.4,0.5,time);
	a=a+makePoint(x,y,1.3,2.1,0.6,0.3,time);
	a=a+makePoint(x,y,1.8,1.7,0.5,0.4,time);

	float b=
	makePoint(x,y,1.2,1.9,0.3,0.3,time);
	b=b+makePoint(x,y,0.7,2.7,0.4,0.4,time);
	b=b+makePoint(x,y,1.4,0.6,0.4,0.5,time);
	b=b+makePoint(x,y,2.6,0.4,0.6,0.3,time);
	b=b+makePoint(x,y,0.7,1.4,0.5,0.4,time);
	b=b+makePoint(x,y,0.7,1.7,0.4,0.4,time);
	b=b+makePoint(x,y,0.8,0.5,0.4,0.5,time);
	b=b+makePoint(x,y,1.4,0.9,0.6,0.3,time);
	b=b+makePoint(x,y,0.7,1.3,0.5,0.4,time);

	float c=
	makePoint(x,y,3.7,0.3,0.3,0.3,time);
	c=c+makePoint(x,y,1.9,1.3,0.4,0.4,time);
	c=c+makePoint(x,y,0.8,0.9,0.4,0.5,time);
	c=c+makePoint(x,y,1.2,1.7,0.6,0.3,time);
	c=c+makePoint(x,y,0.3,0.6,0.5,0.4,time);
	c=c+makePoint(x,y,0.3,0.3,0.4,0.4,time);
	c=c+makePoint(x,y,1.4,0.8,0.4,0.5,time);
	c=c+makePoint(x,y,0.2,0.6,0.6,0.3,time);
	c=c+makePoint(x,y,1.3,0.5,0.5,0.4,time);

	vec3 d=(vec3(a,b,c)-16.0)/32.0;
	d=clamp(vec3(0.),vec3(1.0),d);

	float4 outColor = tex(inputTex, fsTexture);

	float combinedBlendValue=u.blendValue*vars.blendValue;
#ifdef USE_MASK
	blendValue*=tex(inputTexMask,in.texMask).a;
#endif
    d*=combinedBlendValue;
    outColor.rgb=(vec3(1.0)-((vec3(1.0)-outColor.rgb)*(vec3(1.0)-d)));

	return outColor;
}
